@@ -49,11 +49,12 @@ bool MultipleInheritanceCheck::getInterfaceStatus(const CXXRecordDecl *Node,
 bool MultipleInheritanceCheck::isCurrentClassInterface(
     const CXXRecordDecl *Node) const {
   // Interfaces should have no fields.
-  if (!Node->field_empty()) return false;
+  if (!Node->field_empty())
+    return false;
 
   // Interfaces should have exclusively pure methods.
   return llvm::none_of(Node->methods(), [](const CXXMethodDecl *M) {
-    return M->isUserProvided() && !M->isPure() && !M->isStatic();
+    return M->isUserProvided() && !M->isPureVirtual() && !M->isStatic();
   });
 }
 
@@ -62,17 +63,20 @@ bool MultipleInheritanceCheck::isInterface(const CXXRecordDecl *Node) {
     return false;
 
   // Short circuit the lookup if we have analyzed this record before.
-  bool PreviousIsInterfaceResult;
+  bool PreviousIsInterfaceResult = false;
   if (getInterfaceStatus(Node, PreviousIsInterfaceResult))
     return PreviousIsInterfaceResult;
 
   // To be an interface, all base classes must be interfaces as well.
   for (const auto &I : Node->bases()) {
-    if (I.isVirtual()) continue;
+    if (I.isVirtual())
+      continue;
     const auto *Ty = I.getType()->getAs<RecordType>();
-    if (!Ty) continue;
+    if (!Ty)
+      continue;
     const RecordDecl *D = Ty->getDecl()->getDefinition();
-    if (!D) continue;
+    if (!D)
+      continue;
     const auto *Base = cast<CXXRecordDecl>(D);
     if (!isInterface(Base)) {
       addNodeToInterfaceMap(Node, false);
@@ -93,24 +97,29 @@ void MultipleInheritanceCheck::registerMatchers(MatchFinder *Finder) {
 
 void MultipleInheritanceCheck::check(const MatchFinder::MatchResult &Result) {
   if (const auto *D = Result.Nodes.getNodeAs<CXXRecordDecl>("decl")) {
-    // Check against map to see if if the class inherits from multiple
+    // Check against map to see if the class inherits from multiple
     // concrete classes
     unsigned NumConcrete = 0;
     for (const auto &I : D->bases()) {
-      if (I.isVirtual()) continue;
+      if (I.isVirtual())
+        continue;
       const auto *Ty = I.getType()->getAs<RecordType>();
-      if (!Ty) continue;
+      if (!Ty)
+        continue;
       const auto *Base = cast<CXXRecordDecl>(Ty->getDecl()->getDefinition());
-      if (!isInterface(Base)) NumConcrete++;
+      if (!isInterface(Base))
+        NumConcrete++;
     }
-    
-    // Check virtual bases to see if there is more than one concrete 
+
+    // Check virtual bases to see if there is more than one concrete
     // non-virtual base.
     for (const auto &V : D->vbases()) {
       const auto *Ty = V.getType()->getAs<RecordType>();
-      if (!Ty) continue;
+      if (!Ty)
+        continue;
       const auto *Base = cast<CXXRecordDecl>(Ty->getDecl()->getDefinition());
-      if (!isInterface(Base)) NumConcrete++;
+      if (!isInterface(Base))
+        NumConcrete++;
     }
 
     if (NumConcrete > 1) {
