@@ -11,18 +11,20 @@
 // list(list&& c);
 
 // REQUIRES: has-unix-headers
-// UNSUPPORTED: !libcpp-has-debug-mode, c++03
+// UNSUPPORTED: !libcpp-has-legacy-debug-mode, c++03
 
 #include <list>
 
 #include "check_assertion.h"
 
 int main(int, char**) {
-    std::list<int> l1;
-    l1.push_back(1); l1.push_back(2); l1.push_back(3);
-    std::list<int>::iterator i = l1.begin();
-    std::list<int> l2 = l1;
-    TEST_LIBCPP_ASSERT_FAILURE(l2.erase(i), "list::erase(iterator) called with an iterator not referring to this list");
+  std::list<int> l1;
+  l1.push_back(1);
+  l1.push_back(2);
+  l1.push_back(3);
+  std::list<int>::iterator i = l1.begin();
+  std::list<int> l2          = l1;
+  TEST_LIBCPP_ASSERT_FAILURE(l2.erase(i), "list::erase(iterator) called with an iterator not referring to this list");
 
-    return 0;
+  return 0;
 }
