@@ -11,18 +11,18 @@
 // iterator insert(const_iterator position, const value_type& x);
 
 // REQUIRES: has-unix-headers
-// UNSUPPORTED: !libcpp-has-debug-mode, c++03
+// UNSUPPORTED: !libcpp-has-legacy-debug-mode, c++03
 
 #include <list>
 
 #include "check_assertion.h"
 
 int main(int, char**) {
-    std::list<int> v1(3);
-    std::list<int> v2(3);
-    int i = 4;
-    TEST_LIBCPP_ASSERT_FAILURE(v1.insert(v2.begin(), i),
-                               "list::insert(iterator, x) called with an iterator not referring to this list");
+  std::list<int> v1(3);
+  std::list<int> v2(3);
+  int i = 4;
+  TEST_LIBCPP_ASSERT_FAILURE(
+      v1.insert(v2.begin(), i), "list::insert(iterator, x) called with an iterator not referring to this list");
 
-    return 0;
+  return 0;
 }
